@@ -1,5 +1,5 @@
 /***************************************************************************************************
- * Copyright (c) 2017 - 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2017 - 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,16 +49,6 @@ namespace cutlass {
 namespace epilogue {
 namespace thread {
 
-/////////////////////////////////////////////////////////////////////////////////////////////////
-
-template <typename T>
-struct Identity {
-  CUTLASS_HOST_DEVICE
-  T operator()(T value) const {
-    return value;
-  }
-};
-
 /////////////////////////////////////////////////////////////////////////////////////////////////
 template <typename T>
 struct LinearCombinationGenericParams {
@@ -93,6 +83,39 @@ struct LinearCombinationGenericParams {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 
+// Identity operator
+template <typename T>
+struct Identity {
+  static const bool kIsHeavy=false;
+
+  CUTLASS_HOST_DEVICE
+  T operator()(T value) const {
+    return value;
+  }
+
+  using Params = LinearCombinationGenericParams<T>;
+
+  CUTLASS_HOST_DEVICE
+  T operator()(T const &value, Params const &params_) const {
+    return this->operator()(value);
+  }
+};
+
+template <typename T, int N>
+struct Identity<Array<T, N> > {
+  CUTLASS_HOST_DEVICE
+  Array<T, N> operator()(Array<T, N> const &value) const {
+    return value;
+  }
+
+  using Params = LinearCombinationGenericParams<T>;
+
+  CUTLASS_HOST_DEVICE
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value);
+  }
+};
+
 /// ReLu operator - propagates NaNs
 /// Always put threshold in the right hand side of max to propagate NaN.
 template <typename T>
@@ -160,7 +183,7 @@ struct LeakyReLU {
     Params():
       LinearCombinationGenericParams<T>(),
       leaky_alpha(T(1)) {}
-    
+ 
     CUTLASS_HOST_DEVICE
     Params(
       T alpha,
@@ -194,7 +217,7 @@ struct LeakyReLU<Array<T, N> > {
     Params():
       LinearCombinationGenericParams<T>(),
       leaky_alpha(T(1)) {}
-    
+
     CUTLASS_HOST_DEVICE
     Params(
       T alpha,
@@ -205,21 +228,21 @@ struct LeakyReLU<Array<T, N> > {
 
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs, T const & alpha_recip) const {
+  Array<T, N> operator()(Array<T, N> const &value, T const & alpha_recip) const {
     Array<T, N> y;
     LeakyReLU<T> leaky_op;
 
     CUTLASS_PRAGMA_UNROLL
-    for (int i = 0; i < int(rhs.size()); ++i) {
-      y[i] = leaky_op(rhs[i], alpha_recip);
+    for (int i = 0; i < int(value.size()); ++i) {
+      y[i] = leaky_op(value[i], alpha_recip);
     }
 
     return y;
   }
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs, Params const &params_) const {
-    return this->operator()(rhs, params_.leaky_alpha);
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value, params_.leaky_alpha);
   }
 };
 
@@ -242,13 +265,13 @@ struct Tanh {
 template <typename T, int N>
 struct Tanh<Array<T, N> > {
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs) const {
+  Array<T, N> operator()(Array<T, N> const &value) const {
     Array<T, N> y;
     Tanh<T> tanh_op;
 
     CUTLASS_PRAGMA_UNROLL
     for (int i = 0; i < N; ++i) {
-      y[i] = tanh_op(rhs[i]);
+      y[i] = tanh_op(value[i]);
     }
 
     return y;
@@ -257,8 +280,8 @@ struct Tanh<Array<T, N> > {
   using Params = LinearCombinationGenericParams<T>;
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs, Params const &params_) const {
-    return this->operator()(rhs);
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value);
   }
 };
 
@@ -276,8 +299,8 @@ struct Tanh<Array<half_t, N>> {
   using Params = LinearCombinationGenericParams<T>;
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs, Params const &params_) const {
-    return this->operator()(rhs);
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value);
   }
 };
 
@@ -300,13 +323,13 @@ struct Sigmoid {
 template <typename T, int N>
 struct Sigmoid<Array<T, N> > {
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs) const {
+  Array<T, N> operator()(Array<T, N> const &value) const {
     Array<T, N> y;
     Sigmoid<T> sigmoid_op;
 
     CUTLASS_PRAGMA_UNROLL
     for (int i = 0; i < N; ++i) {
-      y[i] = sigmoid_op(rhs[i]);
+      y[i] = sigmoid_op(value[i]);
     }
 
     return y;
@@ -315,8 +338,8 @@ struct Sigmoid<Array<T, N> > {
   using Params = LinearCombinationGenericParams<T>;
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs, Params const &params_) const {
-    return this->operator()(rhs);
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value);
   }
 };
 
@@ -345,6 +368,7 @@ struct Sigmoid<Array<half_t, N>> {
 
   using Params = LinearCombinationGenericParams<T>;
 
+  CUTLASS_HOST_DEVICE
   Array<T, N> operator()(Array<T, N> const &z, Params const &params_) const {
     return this->operator()(z);
   }
@@ -374,17 +398,17 @@ struct SiLu {
 template <typename T, int N>
 struct SiLu<Array<T, N>> {
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs) const {
+  Array<T, N> operator()(Array<T, N> const &value) const {
     Sigmoid<Array<T, N>> sigmoid_op;
     multiplies<Array<T, N>>     mul;
-    return mul(rhs, sigmoid_op(rhs));
+    return mul(value, sigmoid_op(value));
   }
 
   using Params = LinearCombinationGenericParams<T>;
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs, Params const &params_) const {
-    return this->operator()(rhs);
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value);
   }
 };
 
@@ -434,13 +458,13 @@ struct HardSwish<float> {
 template <typename T, int N>
 struct HardSwish<Array<T, N> > {
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs) const {
+  Array<T, N> operator()(Array<T, N> const &value) const {
     Array<T, N> y;
     HardSwish<T> hardswish_op;
 
     CUTLASS_PRAGMA_UNROLL
     for (int i = 0; i < N; ++i) {
-      y[i] = hardswish_op(rhs[i]);
+      y[i] = hardswish_op(value[i]);
     }
 
     return y;
@@ -459,13 +483,13 @@ struct HardSwish<Array<half_t, N> > {
   using T = half_t;
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs) const {
+  Array<T, N> operator()(Array<T, N> const &value) const {
     minimum<Array<T, N> > mn;
     maximum<Array<T, N> > mx;
     multiplies<Array<T, N> > mul;
     plus<Array<T, N> > add;
- 
-    return mul(mul(mn(mx(add(rhs, T(3)), T(0)), T(6)), rhs), T(0.16666667f));
+
+    return mul(mul(mn(mx(add(value, T(3)), T(0)), T(6)), value), T(0.16666667f));
   }
 
   using Params = LinearCombinationGenericParams<T>;
@@ -491,9 +515,9 @@ struct GELU {
   CUTLASS_HOST_DEVICE
   T operator()(T const &scalar) const {
     return T(cutlass::constants::half<T>() * scalar *
-      (cutlass::constants::one<T>() + (T)erff((float)(scalar / cutlass::constants::root_two<T>()))));
+      (cutlass::constants::one<T>() + (T)erff((float)(scalar * cutlass::constants::half_root_two<T>()))));
   }
-  
+
   using Params = LinearCombinationGenericParams<T>;
 
   CUTLASS_HOST_DEVICE
@@ -507,9 +531,9 @@ struct GELU<float> {
   CUTLASS_HOST_DEVICE
   float operator()(float const &scalar) const {
     return cutlass::constants::half<float>() * scalar *
-      (cutlass::constants::one<float>() + erff( scalar / cutlass::constants::root_two<float>() ));
+      (cutlass::constants::one<float>() + erff( scalar * cutlass::constants::half_root_two<float>() ));
   }
-  
+
   using Params = LinearCombinationGenericParams<float>;
 
   CUTLASS_HOST_DEVICE
@@ -523,9 +547,9 @@ struct GELU<double> {
   CUTLASS_HOST_DEVICE
   double operator()(double const &scalar) const {
     return cutlass::constants::half<double>() * scalar *
-      (cutlass::constants::one<double>() + erf( scalar / cutlass::constants::root_two<double>() ));
+      (cutlass::constants::one<double>() + erf( scalar * cutlass::constants::half_root_two<double>() ));
   }
-  
+
   using Params = LinearCombinationGenericParams<double>;
 
   CUTLASS_HOST_DEVICE
@@ -537,23 +561,23 @@ struct GELU<double> {
 template <typename T, int N>
 struct GELU<Array<T, N> > {
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs) const {
+  Array<T, N> operator()(Array<T, N> const &value) const {
     Array<T, N> y;
     GELU<T> gelu_op;
 
     CUTLASS_PRAGMA_UNROLL
     for (int i = 0; i < N; ++i) {
-      y[i] = gelu_op(rhs[i]);
+      y[i] = gelu_op(value[i]);
     }
 
     return y;
   }
-  
+
   using Params = LinearCombinationGenericParams<T>;
 
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs, Params const &params_) const {
-    return this->operator()(rhs);
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value);
   }
 };
 
@@ -572,7 +596,11 @@ struct GELU_taylor {
   }
 
   using Params = LinearCombinationGenericParams<T>;
-  
+
+  CUTLASS_HOST_DEVICE
+  T operator()(T const &scalar, Params const &params_) const {
+    return this->operator()(scalar);
+  }
 };
 
 template <int N>
@@ -601,25 +629,35 @@ struct GELU_taylor<Array<half_t, N> > {
   }
 
   using Params = LinearCombinationGenericParams<half_t>;
+
+  CUTLASS_HOST_DEVICE
+  Array<half_t, N> operator()(Array<half_t, N> const &value, Params const &params_) const {
+    return this->operator()(value);
+  }
 };
 
 template <typename T, int N>
 struct GELU_taylor<Array<T, N> > {
   static const bool kIsHeavy=true;
   CUTLASS_HOST_DEVICE
-  Array<T, N> operator()(Array<T, N> const &rhs) const {
+  Array<T, N> operator()(Array<T, N> const &value) const {
     Array<T, N> y;
     GELU_taylor<T> gelu_op;
 
     CUTLASS_PRAGMA_UNROLL
     for (int i = 0; i < N; ++i) {
-      y[i] = gelu_op(rhs[i]);
+      y[i] = gelu_op(value[i]);
     }
 
     return y;
   }
-  
+
   using Params = LinearCombinationGenericParams<T>;
+
+  CUTLASS_HOST_DEVICE
+  Array<T, N> operator()(Array<T, N> const &value, Params const &params_) const {
+    return this->operator()(value);
+  }
 };
 
 /// Computes backwards pass for GELU operator assuming d_t is the layer gradient and
